/*
 * writebuffer.c
 *
 * A program for writing output to devices which don't like constant
 * stopping and starting, such as tape drives.  writebuffer is:
 *  Copyright (C) 1997-1998,2000-2001 Ian Jackson <ian@chiark.greenend.org.uk>
 *
 * writebuffer is part of chiark backup, a system for backing up GNU/Linux
 * and other UN*X-compatible machines, as used on chiark.greenend.org.uk.
 * chiark backup is:
 *  Copyright (C) 1997-1998,2000-2001 Ian Jackson <ian@chiark.greenend.org.uk>
 *  Copyright (C) 1999 Peter Maydell <pmaydell@chiark.greenend.org.uk>
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this file; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <sys/time.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>

#include "rwbuffer.h"

const char *progname= "readbuffer";

static size_t waitfill;

int main(int argc, const char *const *argv) {
  int r, writing;

  startup(argv);
  waitfill= (buffersize*3)/4;
  writing=0;
  
  while (!seeneof || used) {
    
    FD_ZERO(&readfds); if (!seeneof && used+1<buffersize) FD_SET(0,&readfds);
    FD_ZERO(&writefds); if (writing) FD_SET(1,&writefds);

    callselect();
    
    if (FD_ISSET(1,&writefds) && !FD_ISSET(0,&readfds) && !used) {
      writing= 0;
      FD_CLR(1,&writefds);
    }

    if (FD_ISSET(0,&readfds)) {
      r= read(0,rp,min(buffersize-1-used,buf+buffersize-rp));
      if (!r) {
        seeneof=1; writing=1;
      } else if (r<0) {
        if (!(errno == EAGAIN || errno == EINTR)) { perror("read"); exit(1); }
      } else {
        used+= r;
        rp+= r;
        if (rp == buf+buffersize) rp=buf;
      }
      if (used > waitfill) writing=1;
    }

    if (FD_ISSET(1,&writefds) && used) {
      r= write(1,wp,min(used,buf+buffersize-wp));
      if (r<=0) {
        if (!(errno == EAGAIN || errno == EINTR)) { perror("write"); exit(1); }
      } else {
        used-= r;
        wp+= r;
        if (wp == buf+buffersize) wp=buf;
      }
    }
  }
  exit(0);
}
